package com.ruoyi.common.utils.file;

import java.io.File;
import java.io.IOException;
import java.nio.file.Paths;
import java.util.*;
import com.ruoyi.common.core.domain.AjaxResult;
import org.apache.commons.io.FilenameUtils;
import org.apache.pdfbox.pdmodel.PDDocument;
import org.apache.pdfbox.pdmodel.PDPage;
import org.apache.pdfbox.pdmodel.PDPageContentStream;
import org.apache.pdfbox.pdmodel.common.PDRectangle;
import org.apache.pdfbox.pdmodel.graphics.image.LosslessFactory;
import org.apache.pdfbox.pdmodel.graphics.image.PDImageXObject;
import org.springframework.web.multipart.MultipartFile;
import com.ruoyi.common.config.RuoYiConfig;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.exception.file.FileNameLengthLimitExceededException;
import com.ruoyi.common.exception.file.FileSizeLimitExceededException;
import com.ruoyi.common.exception.file.InvalidExtensionException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.uuid.Seq;
import javax.imageio.ImageIO;

/**
 * 文件上传工具类
 *
 * @author ruoyi
 */
public class FileUploadUtils {
    /**
     * 默认大小 50M
     */
    public static final long DEFAULT_MAX_SIZE = 500 * 1024 * 1024;

    /**
     * 默认的文件名最大长度 100
     */
    public static final int DEFAULT_FILE_NAME_LENGTH = 100;

    /**
     * 默认上传的地址
     */
    private static String defaultBaseDir = RuoYiConfig.getProfile();

    public static void setDefaultBaseDir(String defaultBaseDir) {
        FileUploadUtils.defaultBaseDir = defaultBaseDir;
    }

    public static String getDefaultBaseDir() {
        return defaultBaseDir;
    }

    /**
     * 以默认配置进行文件上传
     *
     * @param file 上传的文件
     * @return 文件名称
     * @throws Exception
     */
    public static final String upload(MultipartFile file) throws IOException {
        try {
            return upload(getDefaultBaseDir(), file, MimeTypeUtils.DEFAULT_ALLOWED_EXTENSION);
        } catch (Exception e) {
            throw new IOException(e.getMessage(), e);
        }
    }

    /**
     * 根据文件路径上传
     *
     * @param baseDir 相对应用的基目录
     * @param file    上传的文件
     * @return 文件名称
     * @throws IOException
     */
    public static final String upload(String baseDir, MultipartFile file) throws IOException {
        try {
            return upload(baseDir, file, MimeTypeUtils.DEFAULT_ALLOWED_EXTENSION);
        } catch (Exception e) {
            throw new IOException(e.getMessage(), e);
        }
    }

    /**
     * 根据文件路径上传 合同编码
     *
     * @param baseDir               相对应用的基目录
     * @param file                  上传的文件
     * @param loanApplicationNumber 合同编码
     * @return 文件名称
     * @throws IOException
     */
    public static final String uploadLoanApplicationNumber(String baseDir, MultipartFile file, String loanApplicationNumber, String originalFilename) throws IOException {
        try {
            return uploadLoanApplicationNumber(baseDir, file, loanApplicationNumber, originalFilename, MimeTypeUtils.DEFAULT_ALLOWED_EXTENSION);
        } catch (Exception e) {
            throw new IOException(e.getMessage(), e);
        }
    }

    /**
     * 文件上传
     *
     * @param baseDir          相对应用的基目录
     * @param file             上传的文件
     * @param allowedExtension 上传文件类型
     * @return 返回上传成功的文件名
     * @throws FileSizeLimitExceededException       如果超出最大大小
     * @throws FileNameLengthLimitExceededException 文件名太长
     * @throws IOException                          比如读写文件出错时
     * @throws InvalidExtensionException            文件校验异常
     */
    public static final String upload(String baseDir, MultipartFile file, String[] allowedExtension)
            throws FileSizeLimitExceededException, IOException, FileNameLengthLimitExceededException,
            InvalidExtensionException {
        int fileNamelength = Objects.requireNonNull(file.getOriginalFilename()).length();
        if (fileNamelength > FileUploadUtils.DEFAULT_FILE_NAME_LENGTH) {
            throw new FileNameLengthLimitExceededException(FileUploadUtils.DEFAULT_FILE_NAME_LENGTH);
        }

        assertAllowed(file, allowedExtension);

        String fileName = extractFilename(file);

        String absPath = getAbsoluteFile(baseDir, fileName).getAbsolutePath();
        file.transferTo(Paths.get(absPath));
        return getPathFileName(baseDir, fileName);
    }

    /**
     * 文件上传合同编号
     *
     * @param baseDir               相对应用的基目录
     * @param file                  上传的文件
     * @param allowedExtension      上传文件类型
     * @param loanApplicationNumber 合同编码
     * @return 返回上传成功的文件名
     * @throws FileSizeLimitExceededException       如果超出最大大小
     * @throws FileNameLengthLimitExceededException 文件名太长
     * @throws IOException                          比如读写文件出错时
     * @throws InvalidExtensionException            文件校验异常
     */
    public static final String uploadLoanApplicationNumber(String baseDir, MultipartFile file, String loanApplicationNumber, String originalFilename, String[] allowedExtension)
            throws FileSizeLimitExceededException, IOException, FileNameLengthLimitExceededException,
            InvalidExtensionException {
        int fileNamelength = Objects.requireNonNull(originalFilename).length();
        if (fileNamelength > FileUploadUtils.DEFAULT_FILE_NAME_LENGTH) {
            throw new FileNameLengthLimitExceededException(FileUploadUtils.DEFAULT_FILE_NAME_LENGTH);
        }

        assertAllowed(file, allowedExtension);

        String fileName = extractFilenameLoanApplicationNumber(file, loanApplicationNumber, originalFilename);

        String absPath = getAbsoluteFile(baseDir, fileName).getAbsolutePath();
        file.transferTo(Paths.get(absPath));
        return getPathFileName(baseDir, fileName);
    }

    /**
     * 编码文件名
     */
    public static final String extractFilename(MultipartFile file) {
        return StringUtils.format("{}/{}_{}.{}", DateUtils.datePath(),
                FilenameUtils.getBaseName(file.getOriginalFilename()), Seq.getId(Seq.uploadSeqType), getExtension(file));
    }

    /**
     * 编码文件名根据合同编号
     */
    public static final String extractFilenameLoanApplicationNumber(MultipartFile file, String loanApplicationNumber, String originalFilename) {
        return StringUtils.format("{}/{}_{}.{}", loanApplicationNumber,
                FilenameUtils.getBaseName(originalFilename), Seq.getId(Seq.uploadSeqType), getExtension(file));
    }

    /**
     * 编码文件名根据合同编号
     */
    public static final String extractFilenameLoanApplicationNumberNoHzm(String originalFilename) {
        return StringUtils.format("{}_{}",
                FilenameUtils.getBaseName(originalFilename), Seq.getId(Seq.uploadSeqType));
    }

    public static final File getAbsoluteFile(String uploadDir, String fileName) throws IOException {
        File desc = new File(uploadDir + File.separator + fileName);

        if (!desc.exists()) {
            if (!desc.getParentFile().exists()) {
                desc.getParentFile().mkdirs();
            }
        }
        return desc;
    }

    public static final String getPathFileName(String uploadDir, String fileName) throws IOException {
        int dirLastIndex = RuoYiConfig.getProfile().length() + 1;
        String currentDir = StringUtils.substring(uploadDir, dirLastIndex);
        return Constants.RESOURCE_PREFIX + "/" + currentDir + "/" + fileName;
    }

    /**
     * 文件大小校验
     *
     * @param file 上传的文件
     * @return
     * @throws FileSizeLimitExceededException 如果超出最大大小
     * @throws InvalidExtensionException
     */
    public static final void assertAllowed(MultipartFile file, String[] allowedExtension)
            throws FileSizeLimitExceededException, InvalidExtensionException {
        long size = file.getSize();
        if (size > DEFAULT_MAX_SIZE) {
            throw new FileSizeLimitExceededException(DEFAULT_MAX_SIZE / 1024 / 1024);
        }

        String fileName = file.getOriginalFilename();
        String extension = getExtension(file);
        if (allowedExtension != null && !isAllowedExtension(extension, allowedExtension)) {
            if (allowedExtension == MimeTypeUtils.IMAGE_EXTENSION) {
                throw new InvalidExtensionException.InvalidImageExtensionException(allowedExtension, extension,
                        fileName);
            } else if (allowedExtension == MimeTypeUtils.FLASH_EXTENSION) {
                throw new InvalidExtensionException.InvalidFlashExtensionException(allowedExtension, extension,
                        fileName);
            } else if (allowedExtension == MimeTypeUtils.MEDIA_EXTENSION) {
                throw new InvalidExtensionException.InvalidMediaExtensionException(allowedExtension, extension,
                        fileName);
            } else if (allowedExtension == MimeTypeUtils.VIDEO_EXTENSION) {
                throw new InvalidExtensionException.InvalidVideoExtensionException(allowedExtension, extension,
                        fileName);
            } else {
                throw new InvalidExtensionException(allowedExtension, extension, fileName);
            }
        }
    }

    /**
     * 判断MIME类型是否是允许的MIME类型
     *
     * @param extension
     * @param allowedExtension
     * @return
     */
    public static final boolean isAllowedExtension(String extension, String[] allowedExtension) {
        for (String str : allowedExtension) {
            if (str.equalsIgnoreCase(extension)) {
                return true;
            }
        }
        return false;
    }

    /**
     * 获取文件名的后缀
     *
     * @param file 表单文件
     * @return 后缀名
     */
    public static final String getExtension(MultipartFile file) {
        String extension = FilenameUtils.getExtension(file.getOriginalFilename());
        if (StringUtils.isEmpty(extension)) {
            extension = MimeTypeUtils.getExtension(Objects.requireNonNull(file.getContentType()));
        }
        return extension;
    }

    public static String uploadLoanApplicationNumberOldName(String baseDir, MultipartFile file, String loanApplicationNumber, String originalFilename) throws IOException {
        try {
            return uploadLoanApplicationNumberOldName(baseDir, file, loanApplicationNumber, originalFilename, MimeTypeUtils.DEFAULT_ALLOWED_EXTENSION);
        } catch (Exception e) {
            throw new IOException(e.getMessage(), e);
        }
    }

    private static String uploadLoanApplicationNumberOldName(String baseDir, MultipartFile file, String loanApplicationNumber, String originalFilename, String[] allowedExtension)
            throws FileSizeLimitExceededException, IOException, FileNameLengthLimitExceededException,
            InvalidExtensionException {
        int fileNamelength = Objects.requireNonNull(originalFilename).length();
        if (fileNamelength > FileUploadUtils.DEFAULT_FILE_NAME_LENGTH) {
            throw new FileNameLengthLimitExceededException(FileUploadUtils.DEFAULT_FILE_NAME_LENGTH);
        }

        assertAllowed(file, allowedExtension);

        String fileName = extractFilenameLoanApplicationNumberOldName(file, loanApplicationNumber, originalFilename);

        String absPath = getAbsoluteFile(baseDir, fileName).getAbsolutePath();
        file.transferTo(Paths.get(absPath));
        return getPathFileName(baseDir, fileName);
    }

    private static String extractFilenameLoanApplicationNumberOldName(MultipartFile file, String loanApplicationNumber, String originalFilename) {
        return StringUtils.format("{}/{}.{}", loanApplicationNumber,
                FilenameUtils.getBaseName(originalFilename), getExtension(file));
    }

    /**
     * 合并图片生成PDF保存为文件
     * @return
     */
    public static AjaxResult createPdfFromImages(List<String> imagePaths, File outputFile) throws IOException {
        try (PDDocument document = new PDDocument()) {
            float margin = 50f; // Define the margin

            for (String imagePath : imagePaths) {
                // Load image
                PDImageXObject image = LosslessFactory.createFromImage(document, ImageIO.read(new File(imagePath)));

                // Calculate scaling factor to fit image within the page minus margins, maintaining aspect ratio
                float imgWidth = image.getWidth();
                float imgHeight = image.getHeight();
                float pageWidthForImage = PDRectangle.A4.getWidth() - 2 * margin; // Subtracting total margin from width
                float pageHeightForImage = PDRectangle.A4.getHeight() - 2 * margin; // Subtracting total margin from height

                float scaleX = pageWidthForImage / imgWidth;
                float scaleY = pageHeightForImage / imgHeight;
                float scale = Math.min(scaleX, scaleY);

                // Resize the image dimensions with the scale
                imgWidth *= scale;
                imgHeight *= scale;

                // Create a new page and add it to the document
                PDPage page = new PDPage(PDRectangle.A4);
                document.addPage(page);

                // Get the content stream for the page
                try (PDPageContentStream contentStream = new PDPageContentStream(document, page)) {
                    // Calculate the centered position considering the total margin around the page
                    float x = (page.getMediaBox().getWidth() - imgWidth) / 2; // Centered without adding margin again
                    float y = (page.getMediaBox().getHeight() - imgHeight) / 2; // Centered without adding margin again

                    // Draw the scaled image onto the page at the correctly centered position
                    contentStream.drawImage(image, x, y, imgWidth, imgHeight); // Apply margin here
                }
            }

            // Save the final document
            document.save(outputFile);
        }
        return null;
    }
}
